@@ -4,6 +4,7 @@ import (
 	"fmt"
 	"os"
 	"strings"
+	"sync"
 
 	configv1 "github.com/openshift/api/config/v1"
 	operatorv1 "github.com/openshift/api/operator/v1"
@@ -31,6 +32,12 @@ type Config struct {
 	Status            ConfigStatus `json:"status,omitempty"`
 }
 
+var (
+	// conditionUpsertLock helps us avoid duplicate entries in the condition array when mutliple
+	// events come in concurrently ... most noticeable on the secret watch and ImportCredentialsExists
+	conditionUpsertLock = sync.Mutex{}
+)
+
 const (
 	// SamplesRegistryCredentials is the name for a secret that contains a username+password/token
 	// for the registry, where if the secret is present, will be used for authentication.
@@ -155,15 +162,16 @@ const (
 	// in the Reason field of the condition.  The Reason field being empty corresponds
 	// with this condition being marked true.
 	ImageChangesInProgress ConfigConditionType = "ImageChangesInProgress"
-	// RemovePending represents whether the Config ManagementState
-	// has been set to Removed while a samples creation/update cycle is still in progress.  In other
-	// words, when ImageChangesInProgress is True.  We
-	// do not want to the create/updates and deletes of the samples to be occurring in parallel.
+	// RemovePending represents whether the Config Spec ManagementState
+	// has been set to Removed, but we have not completed the deletion of the
+	// samples, pull secret, etc. and set the Config Spec ManagementState to Removed.
+	// Also note, while a samples creation/update cycle is still in progress, and ImageChagesInProgress
+	// is True, the operator will not initiate the deletions, as we
+	// do not want the create/updates and deletes of the samples to be occurring in parallel.
 	// So the actual Removed processing will be initated only after ImageChangesInProgress is set
-	// to false.  NOTE:  the optimistic update contention between the imagestream watch trying to
-	// update ImageChangesInProgress and the sampleresource watch simply returning an error an initiating
-	// a retry when ManagementState was set to Removed lead to a prolonged, sometimes seemingly unresolved,
-	// period of circular contention
+	// to false.  Once the deletions are done, and the Status ManagementState is Removed, this
+	// condition is set back to False.  Lastly, when this condition is set to True, the
+	// ClusterOperator Progressing condition will be set to True.
 	RemovePending ConfigConditionType = "RemovePending"
 	// MigrationInProgress represents the special case where the operator is running off of
 	// a new version of its image, and samples are deployed of a previous version.  This condition
@@ -252,6 +260,9 @@ func (s *Config) ConditionsMessages() string {
 }
 
 func (s *Config) ConditionUpdate(c *ConfigCondition) {
+	conditionUpsertLock.Lock()
+	defer conditionUpsertLock.Unlock()
+
 	if s.Status.Conditions == nil {
 		return
 	}
@@ -264,6 +275,9 @@ func (s *Config) ConditionUpdate(c *ConfigCondition) {
 }
 
 func (s *Config) Condition(c ConfigConditionType) *ConfigCondition {
+	conditionUpsertLock.Lock()
+	defer conditionUpsertLock.Unlock()
+
 	if s.Status.Conditions != nil {
 		for _, rc := range s.Status.Conditions {
 			if rc.Type == c {
@@ -311,6 +325,7 @@ const (
 	noInstallDetailed = "Samples installation in error at %s: %s"
 	installed         = "Samples installation successful at %s"
 	moving            = "Samples processing to %s"
+	removing          = "Deleting samples at %s"
 )
 
 // ClusterOperatorStatusAvailableCondition return values are as follows:
@@ -321,6 +336,13 @@ func (s *Config) ClusterOperatorStatusAvailableCondition() (configv1.ConditionSt
 
 	falseRC := configv1.ConditionFalse
 
+	// after online starter upgrade attempts while this operator was not set to managed,
+	// group arch discussion has decided that we report the Available=true if removed/unmanaged
+	if s.Status.ManagementState == operatorv1.Removed ||
+		s.Status.ManagementState == operatorv1.Unmanaged {
+		return configv1.ConditionTrue, fmt.Sprintf(installed, s.Status.Version)
+	}
+
 	// REMINDER: the intital config is always valid, as this operator generates it;
 	// only config changes after by a human cluster admin after
 	// the initial install result in ConfigurationValid == CondtitionFalse
@@ -403,6 +425,9 @@ func (s *Config) ClusterOperatorStatusProgressingCondition(failingState string,
 	if s.ConditionTrue(ImageChangesInProgress) {
 		return configv1.ConditionTrue, fmt.Sprintf(moving, os.Getenv("RELEASE_VERSION"))
 	}
+	if s.ConditionTrue(RemovePending) {
+		return configv1.ConditionTrue, fmt.Sprintf(removing, s.Status.Version)
+	}
 	if available == configv1.ConditionTrue {
 		return configv1.ConditionFalse, fmt.Sprintf(installed, s.Status.Version)
 	}
